/*
 nRF Letterbox Detector
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 13/01/2018
 Last Modified: 26/01/2018
 
 Transmit a single byte packet when movement is detected on the letterbox using the ADXL345
 
 */
 
// ATtiny24/44/84 Pin map
//
//                             +-\/-+
//                       VCC  1|o   |14  GND
//                       PB0  2|    |13  PA0 ADXL INT1
//                       PB1  3|    |12  PA1 ADXL CE
//                 Reset PB3  4|    |11  PA2 
//                       PB2  5|    |10  PA3 nRF CE
//               nRF CSN PA7  6|    |9   PA4 nRF SCK / ADXL SCL
//     nRF MI / ADXL SDO PA6  7|    |8   PA5 nRF MO / ADXL SDA
//                             +----+

#define F_CPU 1000000 // 1 MHz

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include "setup.c"
#include "mirf.c"

int main(void) {
	setup();
	
	while(1) {
		PORTA |= (1<<SCK); // SCK high for ADXL (clock neg)
		adxl_spi_read(0x32); // Read x, y, z to clear
		adxl_spi_read(0x33);
		adxl_spi_read(0x34);
		adxl_spi_read(0x35);
		adxl_spi_read(0x36);
		adxl_spi_read(0x37);
		adxl_spi_read(ADXL345_INT_SOURCE); // Read interrupt source to clear
		
		sbi(GIMSK, PCIE0); // Enable interrupt
		system_sleep();
		cbi(GIMSK, PCIE0); // Disable interrupt
		
		PORTA &= ~(1<<SCK); // SCK low for nRF (clock pos)
		mirf_transmit_data();
	}
}

ISR(PCINT0_vect) { }
