/*
 nRF Letterbox Detector
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 13/01/2018
 Last Modified: 26/01/2018

 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

// ADXL345
#define ADXL_CE PA1
#define ADXL_CE_hi	PORTA |=  (1<<ADXL_CE);
#define ADXL_CE_lo	PORTA &= ~(1<<ADXL_CE);

#define SCK PA4
#define MOSI PA5
#define MISO PA6

// Definitions from Sparkfun
#define ADXL345_THRESH_ACT    0x24    // Activity Threshold
#define ADXL345_THRESH_INACT  0x25    // Inactivity Threshold
#define ADXL345_TIME_INACT    0x26    // Inactivity Time
#define ADXL345_ACT_INACT_CTL 0x27    // Axis Enable Control for Activity and Inactivity Detection
#define ADXL345_BW_RATE      	0x2C    // Data Rate and Power mode Control
#define ADXL345_INT_ENABLE    0x2E    // Interrupt Enable Control
#define ADXL345_INT_MAP     	0x2F    // Interrupt Mapping Control
#define ADXL345_INT_SOURCE    0x30    // Source of Interrupts
#define ADXL345_POWER_CTL    	0x2D    // Power-Saving Features Control
#define ADXL345_DATA_FORMAT   0x31    // Data Format Control

extern void mirf_init(void);
extern void mirf_config(void);


// AXDL SPI transfer 1 byte and return the result, negative clock
uint8_t adxl_spi_transfer(uint8_t data) {
	uint8_t dataRead = 0;
	for (int8_t i = 7; i >= 0; i--) {
		PORTA &= ~(1<<SCK);
		
		if (data & (1<<i)) { // Output
			PORTA |= (1<<MOSI);
		} 
		else {
			PORTA &= ~(1<<MOSI);
		}
		
		if (PINA & (1<<MISO)) { // Input
			dataRead |= (1<<i);
		}
		
		PORTA |= (1<<SCK);
	}
	
	return dataRead;
}

// ADXL SPI write
void adxl_spi_write(uint8_t reg, uint8_t data) {
  ADXL_CE_lo;
  adxl_spi_transfer(reg); 
  adxl_spi_transfer(data); 
  ADXL_CE_hi;
}

// AXDL SPI read
uint8_t adxl_spi_read(uint8_t address) {
  ADXL_CE_lo;
  address |= 0x80; // For read
  adxl_spi_transfer(address); 
  uint8_t readByte = adxl_spi_transfer(0x00); 
  ADXL_CE_hi;

  return readByte;
}


// nRF SPI transfer 1 byte and return the result, positive clock
uint8_t mirf_spi_transfer(uint8_t data) {
	uint8_t dataRead = 0;
	for (int8_t i = 7; i >= 0; i--) {
		if (data & (1<<i)) { // Output
			PORTA |= (1<<MOSI);
		} 
		else {
			PORTA &= ~(1<<MOSI);
		}
		
		PORTA |= (1<<SCK);
		
		if (PORTA & (1<<MISO)) { // Input
			dataRead |= (1<<i);
		}
		PORTA &= ~(1<<SCK);
	}
	
	return dataRead;
}

// MIRF Write data using SPI
void mirf_spi_write_data(uint8_t * dataout, uint8_t len) {
	for (uint8_t i = 0; i < len; i++) {
		mirf_spi_transfer(dataout[i]);
	}
}

// MIRF Read data using SPI
void mirf_spi_read_data(uint8_t * datain, uint8_t len) {
	for (uint8_t i = 0; i < len; i++) {
		datain[i] = mirf_spi_transfer(0x00);
	}
}


// Initialise the SPI
void spi_init(void) {
	DDRA |= (1<<PA4); // SPI CLK
	DDRA |= (1<<PA5); // SPI DO
	DDRA &= ~(1<<PA6); // SPI DI
	PORTA |= (1<<PA6); // SPI DI
}

// Sleep
void system_sleep(void) {
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
}

// Setup
void setup(void) {
	_delay_ms(250); // Wait a little bit after power on
	spi_init(); // Initialise SPI
	
	// AXDL init
	DDRA |= (1<<ADXL_CE); // CE as output
	ADXL_CE_hi; // CE high
	PORTA |= (1<<SCK); // SCK high
	
	// mIRF initialise and configure
	mirf_init();
	mirf_config();
	
	// Initialise the pin change for AXDL INT1
	sbi(GIMSK, PCIE0);
	sbi(PCMSK0, PCINT0);
	
	// Turn on pull-up resistors on other ports to save power
	PORTB |= ((1<<PB0) | (1<<PB1) | (1<<PB2)); 
	PORTA |= (1<<PA2);
	
	sei(); // Turn on interrupts
	
	
	// ADXL setup
	adxl_spi_write(ADXL345_POWER_CTL, 0); // Wakeup     
	adxl_spi_write(ADXL345_DATA_FORMAT, 0); // 2g and 4-wire SPI
	
	adxl_spi_write(ADXL345_BW_RATE, 0x17); // Low Power Mode with 12.5 Hz data rate
	adxl_spi_write(ADXL345_ACT_INACT_CTL, 0xF0); // Set activity on x, y, z (AC)
	adxl_spi_write(ADXL345_THRESH_ACT, 5); // Activity Threshold
	adxl_spi_write(ADXL345_INT_ENABLE, 0x10); // Enable interrupt on activity
	
	adxl_spi_write(ADXL345_POWER_CTL, 0x08); // Measurement mode
}

